@@ -1,5 +1,5 @@
 # GNU Solfege - free ear training software
-# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008  Tom Cato Amundsen
+# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2011  Tom Cato Amundsen
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -86,18 +86,28 @@ class Teacher(abstract.Teacher):
             return Teacher.ERR_NO_INTERVALLS
         last_question = self.m_interval
         last_tonika = self.m_tonika
-        L, H = utils.adjust_low_high_to_irange(L, H, self.get_list('intervals'))
         while 1:
             # in this loop we will try to make a question that is not the
             # same that the last one.
-            self.m_tonika, self.m_interval = \
-                  utils.random_tonika_and_interval(L, H,
-                    self.get_list('intervals'))
+            try:
+                if self.get_bool("lock-to-key"):
+                    self.m_tonika, self.m_interval = \
+                        utils.random_tonic_and_interval_in_key(L, H,
+                            self.get_list('intervals'),
+                            mpd.MusicalPitch.new_from_int(self.get_int("lock-to-key-note")),
+                            solfege.utils.key_data.keys()[:][self.get_int("lock-to-key-scaletype")])
+                else:
+                    self.m_tonika, self.m_interval = \
+                          utils.random_tonika_and_interval(L, H,
+                            self.get_list('intervals'))
+            except solfege.utils.NoPossibleIntervals, e:
+                solfege.win.display_error_message2(_("Exercise configuration problem"), unicode(e))
+                return self.ERR_NO_INTERVALLS
             if last_question is None:
                 break
             if (self.m_interval == last_question
                 and self.m_tonika == last_tonika) \
-                and (len(self.get_list('intervals')) > 1 or (H - L) > 1):
+                and (len(self.get_list('intervals')) > 1):
                 continue
             break
         assert self.m_tonika
@@ -183,6 +193,8 @@ class Gui(abstract.IntervalGui):
         self.add_watch('intervals', _ff)
         self.g_interval_selector.set_border_width(gu.PAD)
         frame.add(self.g_interval_selector)
+
+        self.add_lock_to_key_gui()
         #------we need some space
         self.config_box.pack_start(gtk.HBox(), False,
                                    padding=gu.PAD_SMALL)
