@@ -503,58 +503,23 @@ class AccordionWidget(gtk.DrawingArea, CairoCommon):
         gtk.DrawingArea.__init__(self)
         CairoCommon.__init__(self)
         self.m_callback = callback
-        self.m_notenames_norwegian = (("g,", "bes,", "des", "e",
-                 "g", "bes", "des'", "e'", "g'", "bes'", "des''", "e''",
-                 "g''", "bes''", "des'''", "e'''", "g'''", "bes'''", "des''''"),
-                ("f,", "aes,", "b,", "d",
-                 "f", "aes", "b", "d'", "f'", "aes'", "b'", "d''", "f''",
-                 "aes''", "b''", "d'''", "f'''", "aes'''", "b'''", "d''''"),
-                ("fis,", "a,", "c", "ees",
-                 "fis", "a", "c'", "ees'", "fis'", "a'", "c''", "ees''",
-                 "fis''", "a''", "c'''", "ees'''", "fis'''", "a'''", "c''''"),
-                ("e,", "g,", "bes,", "des",
-                 "e", "g", "bes", "des'", "e'", "g'", "bes'", "des''", "e''",
-                 "g''", "bes''", "des'''", "e'''", "g'''", "bes'''", "des''''"),
-                ("f,", "aes,", "b,", "d",
-                 "f", "aes", "b", "d'", "f'", "aes'", "b'", "d''", "f''",
-                 "aes''", "b''", "d'''", "f'''", "aes'''", "b'''"))
-        self.m_notenames_swedish = (
-                ("e,", "g,", "bes,", "des",
-                 "e", "g", "bes", "des'", "e'", "g'", "bes'", "des''", "e''",
-                 "g''", "bes''", "des'''", "e'''", "g'''", "bes'''"),
-                ("dis,", "fis,", "a,", "c", "ees", "fis", "a", "c'",
-                 "ees'", "fis'", "a'", "c''", "ees''", "fis''", "a''",
-                 "c'''", "ees'''", "fis'''", "a'''", "c''''"),
-                ("f,", "aes,", "b,", "d",
-                 "f", "aes", "b", "d'", "f'", "aes'", "b'", "d''", "f''",
-                 "aes''", "b''", "d'''", "f'''", "aes'''", "b'''"),
-                ("e,", "g,", "bes,", "des",
-                 "e", "g", "bes", "des'", "e'", "g'", "bes'", "des''", "e''",
-                 "g''", "bes''", "des'''", "e'''", "g'''", "bes'''", "des''''"),
-                ("fis,", "a,", "c", "ees",
-                 "fis", "a", "c'", "ees'", "fis'", "a'", "c''", "ees''",
-                 "fis''", "a''", "c'''", "ees'''", "fis'''", "a'''", "c''''"),
-                 )
-        self.m_notenames_finnish = (
-                ("dis,", "fis,", "a,", "c", "ees", "fis", "a", "c'",
-                 "ees'", "fis'", "a'", "c''", "ees''", "fis''", "a''",
-                 "c'''", "ees'''", "fis'''", "a'''", "c''''"),
-                ("d,", "f,", "aes,", "b,", "d",
-                 "f", "aes", "b", "d'", "f'", "aes'", "b'", "d''", "f''",
-                 "aes''", "b''", "d'''", "f'''", "aes'''", "b'''"),
-                ("e,", "g,", "bes,", "des",
-                 "e", "g", "bes", "des'", "e'", "g'", "bes'", "des''", "e''",
-                 "g''", "bes''", "des'''", "e'''", "g'''", "bes'''", "des''''"),
-                ("dis,", "fis,", "a,", "c", "ees",
-                 "fis", "a", "c'", "ees'", "fis'", "a'", "c''", "ees''",
-                 "fis''", "a''", "c'''", "ees'''", "fis'''", "a'''", "c''''"),
-                ("f,", "aes,", "b,", "d",
-                 "f", "aes", "b", "d'", "f'", "aes'", "b'", "d''", "f''",
-                 "aes''", "b''", "d'''", "f'''", "aes'''", "b'''", "d''''"),
-                 )
-        self.m_notenames = {'norwegian': self.m_notenames_norwegian,
-                            'swedish': self.m_notenames_swedish,
-                       'finnish': self.m_notenames_finnish}[keyboard_system]
+        def ff(first, count):
+            i = mpd.notename_to_int(first)
+            return [mpd.int_to_octave_notename(i + x * 3) for x in range(count)]
+        keyboard_def = {'norwegian':
+            (ff("g,", 19), ff("f,", 20), ff("fis,", 19),
+            ff("e,", 20), ff("f,", 19))}
+        keyboard_def['swedish'] = (
+            ff("e,", 19), ff("dis,", 20), ff("f,", 19),
+            ff("e,", 20), ff("fis,", 19))
+        keyboard_def['finnish'] = (
+            ff("dis,", 19), ff("d,", 20), ff("e,", 19),
+            ff("dis,", 20), ff("f,", 19))
+        keyboard_def['belgian'] = (
+            ff("fis,", 19), ff("e,", 20), ff("f,", 19),
+            ff("es,", 20), ff("e,", 19)
+        )
+        self.m_notenames = keyboard_def[keyboard_system]
         self.m_lowest_tone = mpd.HIGHEST_NOTENAME
         self.m_highest_tone = mpd.LOWEST_NOTENAME
         for v in self.m_notenames:
@@ -658,7 +623,8 @@ inputwidget_names = [_("Buttons"),
                  _("Piano"), _("Guitar"), _("Bass"),
                  _("5 string bass"), _("6 string bass"),
                  _("Accordion B griff"), _("Accordion C griff"),
-                 _("Accordion (system used in Finland)")]
+                 _("Accordion (system used in Finland)"),
+                 _("Accordion (Belgian Charleroi do2)")]
 
 def name_to_inputwidget(name, callback):
     if name == _("Piano"):
@@ -669,6 +635,8 @@ def name_to_inputwidget(name, callback):
         w = AccordionWidget(callback, 'swedish')
     elif name == _("Accordion (system used in Finland)"):
         w = AccordionWidget(callback, 'finnish')
+    elif name == _("Accordion (Belgian Charleroi do2)"):
+        w = AccordionWidget(callback, 'belgian')
     elif name == _("Guitar"):
         w = GuitarWidget(callback, ["e'", "b", "g", "d", "a,", "e,"],
                                    (1, 1, 1, 2, 2, 3))
