@@ -1,6 +1,6 @@
 # GNU Solfege - free ear training software
 # vim: set fileencoding=utf-8 :
-# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 Tom Cato Amundsen
+# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2011 Tom Cato Amundsen
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -149,7 +149,7 @@ class DB(object):
         assert lessonfile.is_uri(filename) or os.path.isabs(filename), filename
         row = self.conn.execute(
                 "select fileid from lessonfiles "
-                "where filename=?", 
+                "where filename=?",
                 (filename,)).fetchone()
         if row:
             return row[0]
@@ -169,7 +169,7 @@ class DB(object):
             mapping[uuid] = lessonfile.mk_uri(data['filename'])
             count += 1
             callback(_("Files read: %i") % count)
-        logging.info("get_uuid_to_filename_mapping: scanned %i files." % count)
+        logging.info("get_uuid_to_filename_mapping: scanned %i files.", count)
         return count, mapping
     def read_old_data(self, callback):
         """
@@ -197,12 +197,12 @@ class DB(object):
         def load(file):
             unpickler = pickle.Unpickler(file)
             unpickler.dispatch[pickle.GLOBAL] = mapped_load_global
-            return unpickler.load() 
+            return unpickler.load()
 
         imp, uuid_to_filename = self.get_uuid_to_filename_mapping(callback)
         def read_session_data(datadir, is_tests, counter):
             if not os.path.isdir(datadir):
-                logging.debug("read_session_data: '%s' does not exist. Returning" % datadir)
+                logging.debug("read_session_data: '%s' does not exist. Returning", datadir)
                 return
             for uuid in os.listdir(datadir):
                 if not (os.path.isdir(os.path.join(datadir, uuid))
@@ -256,7 +256,7 @@ class DB(object):
 
                     if not (saved_hashvalue == bugfix_hash_file(real_lessonfilename, "hash-bug-workaround")
                             or saved_hashvalue == bugfix_hash_file(real_lessonfilename, "hash-bug-workaround2")):
-                        logging.debug("Ignoring statistics for '%s' since the saved hash value does not match." % lessonfilename)
+                        logging.debug("Ignoring statistics for '%s' since the saved hash value does not match.", lessonfilename)
                         continue
 
                 for timestamp in os.listdir(os.path.join(datadir, uuid)):
@@ -301,11 +301,11 @@ class DB(object):
                 os.path.join(filesystem.app_data(), 'testresults'),
                 True, imp)
         self.set_variable("database_version", 2)
-        logging.info("imported statistics for %s lessonfiles" % imp)
+        logging.info("imported statistics for %s lessonfiles", imp)
         self.conn.commit()
     def upgrade_to_version_2(self):
         """
-        After releaseing 3.16.0 I found that having separate tests and
+        After releasing 3.16.0 I found that having separate tests and
         sessions table was a bad idea since we may want to add different
         tests later. Also tests and practise sessions in one table make
         it even easier to see which questions needs most practise.
@@ -325,7 +325,7 @@ class DB(object):
                     "values (?, ?, ?)",
                     (fileid, timestamp, 1))
             except sqlite3.IntegrityError, e:
-                logging.error(u"%s, %s, %s" % (e, fileid, timestamp))
+                logging.error(u"%s, %s, %s", e, fileid, timestamp)
         # Insert all practise sessions into sessioninfo
         for fileid, timestamp in self.conn.execute("select distinct fileid, timestamp from sessions"):
             # This can fail because 3.14.11 and earlier had a bug where the same
@@ -336,7 +336,7 @@ class DB(object):
                     "values (?, ?, ?)",
                     (fileid, timestamp, 0))
             except sqlite3.IntegrityError, e:
-                logging.error(u"%s, %s, %s" % (e, fileid, timestamp))
+                logging.error(u"%s, %s, %s", e, fileid, timestamp)
         # Copy all tests from the obsolete "tests" table to "sessions"
         for a, b, c, d in self.conn.execute("select fileid, timestamp, answerkey, guessed from tests"):
             # This can fail because 3.14.11 and earlier had a bug where the same
@@ -347,7 +347,7 @@ class DB(object):
                     "values (?, ?, ?, ?)",
                     (a, b, c, d))
             except sqlite3.IntegrityError, e:
-                logging.error(u"%s, %s, %s" % (e, fileid, timestamp))
+                logging.error(u"%s, %s, %s", e, fileid, timestamp)
         for fileid, filename in self.conn.execute(
                 "select sessioninfo.fileid, lessonfiles.filename "
                 "from sessioninfo, lessonfiles "
@@ -368,14 +368,14 @@ class DB(object):
                     self.cache_new_test_result(filename, timestamp,
                         p.get_test_requirement(), p.get_test_num_questions())
         self.set_variable("database_version", 2)
-    def cache_new_test_result(self, filename, timestamp, 
+    def cache_new_test_result(self, filename, timestamp,
                               required, num_questions):
         """
         Save the test result in the "lessonfiles" table for faster
         access later.
         """
-        logging.debug("cache_new_test_result(%s, %s, %s, %s)"
-            % (filename, timestamp, required, num_questions))
+        logging.debug("cache_new_test_result(%s, %s, %s, %s)",
+            filename, timestamp, required, num_questions)
         fileid = self.get_fileid(filename)
         count_correct = self.conn.execute("select sum(count) from sessions where fileid=? and timestamp=? and answerkey=guessed", (fileid, timestamp)).fetchone()[0]
         if not count_correct:
@@ -404,7 +404,7 @@ class DB(object):
         (bool, float)
         If no test have been made, we return (None, None)
         """
-        logging.debug("get_test_status(%s)" % filename)
+        logging.debug("get_test_status(%s)", filename)
         try:
             fileid = self.get_fileid(filename)
             row = self.conn.execute("select test_passed, test_result "
@@ -424,7 +424,7 @@ class DB(object):
 
         Return doing nothing if the file does not exist.
         """
-        logging.debug("validate_stored_statistics(%s)" % filename)
+        logging.debug("validate_stored_statistics(%s)", filename)
         if not os.path.isfile(lessonfile.uri_expand(filename)):
             logging.debug("validate_stored_statistics: file does not exist.")
             return
@@ -445,8 +445,7 @@ class DB(object):
                     replaces = [replaces]
                 if not hashvalue in replaces:
                     cursor.execute("delete from sessions where fileid=?", (fileid,))
-                    cursor.execute("delete from tests where fileid=?", (fileid,))
-                    cursor.execute("delete from sessioninfo where fileid=?", (fileid,)) 
+                    cursor.execute("delete from sessioninfo where fileid=?", (fileid,))
                     cursor.execute("update lessonfiles "
                         "set hash=?, test_passed=?, test_result=0.0 where fileid=?",
                         (cur_lessonfile_hash_value, None, fileid))
@@ -527,7 +526,7 @@ class DB(object):
         """
         cursor = self.conn.execute('delete from variables where variable_name=?', (name,))
         if not cursor.rowcount:
-            # rowcount is 0 if no variables where deleted. We raise an 
+            # rowcount is 0 if no variables where deleted. We raise an
             # error, since I'd like to know if we tries to do this.
             raise self.VariableUndefinedError()
 
@@ -592,14 +591,23 @@ class AbstractStatistics(object):
             q = solfege.db.conn.execute("select answerkey, guessed, sum(count) from sessions where fileid=? and timestamp>? group by answerkey, guessed", (fileid, self.m_timestamp - seconds))
         ret = {}
         for answer, guess, count in q.fetchall():
-            ret.setdefault(self.int_if_int(answer), {})
-            ret[self.int_if_int(answer)][self.int_if_int(guess)] = count
+            ret.setdefault(answer, {})
+            ret[answer][guess] = count
         return ret
     def reset_session(self):
         """
         Start a new practise session.
         """
         self.m_timestamp = int(time.time())
+    def reset_custom_mode_session(self, filename):
+        """
+        Exercises that show statistics in custom_mode should call this
+        method instead of reset_session.
+        """
+        self.m_timestamp = 1
+        fileid = solfege.db.get_fileid(filename)
+        solfege.db.conn.execute("delete from sessions "
+                          "where fileid=? and timestamp=1", (fileid,))
     def enter_test_mode(self):
         self.m_test_mode = True
     def exit_test_mode(self):
@@ -619,7 +627,9 @@ class AbstractStatistics(object):
         Register that for the question 'question' the user answered 'answer'.
         """
         assert self.m_timestamp
-        # tuples must be converted to str to store them in sqlite.
+        # tuples must be converted to str to store them in sqlite. Integers
+        # and probably some other types will automatically be converted to
+        # strings by sqlite before storing then in the database.
         if isinstance(question, tuple):
             question = str(question)
         if isinstance(answer, tuple):
@@ -645,7 +655,7 @@ class AbstractStatistics(object):
                 "insert into sessions "
                 "(fileid, timestamp, answerkey, guessed, count) "
                 "values(?, ?, ?, ?, ?)",
-                (fileid, self.m_timestamp, 
+                (fileid, self.m_timestamp,
                  unicode(question), unicode(answer), 1))
         else:
             assert cursor.fetchone() is None
@@ -760,7 +770,9 @@ class IntervalStatistics(AbstractStatistics):
     def get_keys(self, all_keys=False):
         # FIXME we have to check that all keys are integers, and filter
         # out those that are not. I don't know how, but I ended up with
-        # the key None inserted into the database.
+        # the key None inserted into the database. Without that bug this
+        # whole method would be unnecessary and AbstractStatistics.get_keys
+        # would be enough.
         def isinteger(s):
             try:
                 int(s)
@@ -768,7 +780,7 @@ class IntervalStatistics(AbstractStatistics):
             except ValueError:
                 return False
         v = AbstractStatistics.get_keys(self, all_keys)
-        return [int(x) for x in v if isinteger(x)]
+        return [x for x in v if isinteger(x)]
     def key_to_pretty_name(self, key):
         return utils.int_to_intervalname(int(key), 1, 1)
 
