@@ -952,6 +952,24 @@ class TestLabelObject(unittest.TestCase):
             ):
             self.assertEquals(chordname_markup_tokenizer(s), v)
 
+class TestOurLessonFiles(unittest.TestCase):
+    def test_test_requirement(self):
+        """
+        Check that all lesson files that has the header.test variable
+        also defines header.test_requirement
+        """
+        d = os.path.join("exercises", "standard", "lesson-files")
+        for fn in os.listdir(d):
+            if fn == 'Makefile':
+                continue
+            fullname = os.path.join(d, fn)
+            if os.path.isfile(fullname):
+                if fn.endswith("~") or fn.startswith("."):
+                    continue
+                f = parse_lesson_file_header(fullname)
+                self.failIf(f is None, "Parsing lesson header failed: %s" % fullname)
+                if 'test' in f.header:
+                    self.failIf('test_requirement' not in f.header, fullname)
 
 suite = unittest.makeSuite(TestParser)
 suite.addTest(unittest.makeSuite(TestMpdTransposable))
@@ -974,3 +992,4 @@ suite.addTest(unittest.makeSuite(TestLessonfileQuestion))
 suite.addTest(unittest.makeSuite(TestLessonfileMisc))
 suite.addTest(unittest.makeSuite(TestErrorHandling))
 suite.addTest(unittest.makeSuite(TestLabelObject))
+suite.addTest(unittest.makeSuite(TestOurLessonFiles))
